@@ -2,9 +2,7 @@ use self::hashing::*;
 use crate::prelude::*;
 #[cfg(feature = "groupby_list")]
 use crate::utils::Wrap;
-use crate::utils::{
-    accumulate_dataframes_vertical, copy_from_slice_unchecked, set_partition_size, split_offsets,
-};
+use crate::utils::{accumulate_dataframes_vertical, set_partition_size, split_offsets};
 use crate::vector_hasher::{get_null_hash_value, AsU64, StrHash};
 use crate::POOL;
 use ahash::{CallHasher, RandomState};
@@ -36,6 +34,12 @@ impl DataFrame {
         multithreaded: bool,
         sorted: bool,
     ) -> Result<GroupBy> {
+        if by.is_empty() {
+            return Err(PolarsError::ComputeError(
+                "expected keys in groupby operation, got nothing".into(),
+            ));
+        }
+
         macro_rules! finish_packed_bit_path {
             ($ca0:expr, $ca1:expr, $pack_fn:expr) => {{
                 let n_partitions = set_partition_size();
@@ -144,7 +148,7 @@ impl DataFrame {
 
                     // arbitrarily chosen bound, if avg no of bytes to encode is larger than this
                     // value we fall back to default groupby
-                    if (lhs.get_values_size() + rhs.get_values_size()) / lhs.len() < 128 {
+                    if (lhs.get_values_size() + rhs.get_values_size()) / (lhs.len() + 1) < 128 {
                         pack_utf8_columns(lhs, rhs, n_partitions, sorted)
                     } else {
                         groupby_threaded_multiple_keys_flat(keys_df, n_partitions, sorted)
@@ -301,16 +305,27 @@ impl<'df> GroupBy<'df> {
         self.groups
     }
 
-    pub fn keys(&self) -> Vec<Series> {
+    pub fn keys_sliced(&self, slice: Option<(i64, usize)>) -> Vec<Series> {
         POOL.install(|| {
             self.selected_keys
                 .par_iter()
                 .map(|s| {
+                    #[allow(unused_assignments)]
+                    // needed to keep the lifetimes valid for this scope
+                    let mut groups_owned = None;
+
+                    let groups = if let Some((offset, len)) = slice {
+                        groups_owned = Some(self.groups.slice(offset, len));
+                        groups_owned.as_deref().unwrap()
+                    } else {
+                        &self.groups
+                    };
+
                     // Safety
                     // groupby indexes are in bound.
                     unsafe {
                         s.take_iter_unchecked(
-                            &mut self.groups.idx_ref().iter().map(|(idx, _)| idx as usize),
+                            &mut groups.idx_ref().iter().map(|(idx, _)| idx as usize),
                         )
                     }
                 })
@@ -318,6 +333,10 @@ impl<'df> GroupBy<'df> {
         })
     }
 
+    pub fn keys(&self) -> Vec<Series> {
+        self.keys_sliced(None)
+    }
+
     fn prepare_agg(&self) -> Result<(Vec<Series>, Vec<Series>)> {
         let selection = match &self.selected_agg {
             Some(selection) => selection.clone(),
@@ -951,7 +970,7 @@ impl<'df> GroupBy<'df> {
     }
 }
 
-#[derive(Copy, Clone)]
+#[derive(Copy, Clone, Debug)]
 pub enum GroupByMethod {
     Min,
     Max,
@@ -1001,7 +1020,7 @@ mod test {
     #[cfg(feature = "dtype-date")]
     #[cfg_attr(miri, ignore)]
     fn test_group_by() -> Result<()> {
-        let s0 = DateChunked::parse_from_str_slice(
+        let s0 = Series::new(
             "date",
             &[
                 "2020-08-21",
@@ -1010,9 +1029,7 @@ mod test {
                 "2020-08-23",
                 "2020-08-22",
             ],
-            "%Y-%m-%d",
-        )
-        .into_series();
+        );
         let s1 = Series::new("temp", [20, 10, 7, 9, 1]);
         let s2 = Series::new("rain", [0.2, 0.1, 0.3, 0.1, 0.01]);
         let df = DataFrame::new(vec![s0, s1, s2]).unwrap();
