@@ -2,9 +2,8 @@ use super::private;
 use super::IntoSeries;
 use super::SeriesTrait;
 use super::SeriesWrap;
-use crate::chunked_array::{
-    comparison::*, ops::explode::ExplodeByOffsets, AsSinglePtr, ChunkIdIter,
-};
+use super::*;
+use crate::chunked_array::{ops::explode::ExplodeByOffsets, AsSinglePtr};
 use crate::fmt::FmtList;
 use crate::frame::{groupby::*, hash_join::*};
 use crate::prelude::*;
@@ -58,20 +57,10 @@ impl private::PrivateSeries for SeriesWrap<DatetimeChunked> {
             .into_series()
     }
 
-    #[cfg(feature = "asof_join")]
-    fn join_asof(&self, other: &Series) -> Result<Vec<Option<u32>>> {
-        let other = other.to_physical_repr();
-        self.0.deref().join_asof(&other)
-    }
-
     fn set_sorted(&mut self, reverse: bool) {
         self.0.deref_mut().set_sorted(reverse)
     }
 
-    unsafe fn equal_element(&self, idx_self: usize, idx_other: usize, other: &Series) -> bool {
-        self.0.equal_element(idx_self, idx_other, other)
-    }
-
     #[cfg(feature = "zip_with")]
     fn zip_with_same_type(&self, mask: &BooleanChunked, other: &Series) -> Result<Series> {
         let other = other.to_physical_repr().into_owned();
@@ -89,59 +78,36 @@ impl private::PrivateSeries for SeriesWrap<DatetimeChunked> {
         self.0.vec_hash_combine(build_hasher, hashes)
     }
 
-    fn agg_mean(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
-        // does not make sense on logical
-        None
-    }
-
-    fn agg_min(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+    fn agg_min(&self, groups: &GroupsProxy) -> Option<Series> {
         self.0.agg_min(groups).map(|ca| {
             ca.into_datetime(self.0.time_unit(), self.0.time_zone().clone())
                 .into_series()
         })
     }
 
-    fn agg_max(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+    fn agg_max(&self, groups: &GroupsProxy) -> Option<Series> {
         self.0.agg_max(groups).map(|ca| {
             ca.into_datetime(self.0.time_unit(), self.0.time_zone().clone())
                 .into_series()
         })
     }
 
-    fn agg_sum(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+    fn agg_sum(&self, _groups: &GroupsProxy) -> Option<Series> {
         // does not make sense on logical
         None
     }
 
-    fn agg_first(&self, groups: &[(u32, Vec<u32>)]) -> Series {
-        self.0
-            .agg_first(groups)
-            .into_datetime(self.0.time_unit(), self.0.time_zone().clone())
-            .into_series()
-    }
-
-    fn agg_last(&self, groups: &[(u32, Vec<u32>)]) -> Series {
-        self.0
-            .agg_last(groups)
-            .into_datetime(self.0.time_unit(), self.0.time_zone().clone())
-            .into_series()
-    }
-
-    fn agg_std(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+    fn agg_std(&self, _groups: &GroupsProxy) -> Option<Series> {
         // does not make sense on logical
         None
     }
 
-    fn agg_var(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+    fn agg_var(&self, _groups: &GroupsProxy) -> Option<Series> {
         // does not make sense on logical
         None
     }
 
-    fn agg_n_unique(&self, groups: &[(u32, Vec<u32>)]) -> Option<UInt32Chunked> {
-        self.0.agg_n_unique(groups)
-    }
-
-    fn agg_list(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+    fn agg_list(&self, groups: &GroupsProxy) -> Option<Series> {
         // we cannot cast and dispatch as the inner type of the list would be incorrect
         self.0.agg_list(groups).map(|s| {
             s.cast(&DataType::List(Box::new(self.dtype().clone())))
@@ -151,7 +117,7 @@ impl private::PrivateSeries for SeriesWrap<DatetimeChunked> {
 
     fn agg_quantile(
         &self,
-        groups: &[(u32, Vec<u32>)],
+        groups: &GroupsProxy,
         quantile: f64,
         interpol: QuantileInterpolOptions,
     ) -> Option<Series> {
@@ -161,32 +127,16 @@ impl private::PrivateSeries for SeriesWrap<DatetimeChunked> {
         })
     }
 
-    fn agg_median(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+    fn agg_median(&self, groups: &GroupsProxy) -> Option<Series> {
         self.0.agg_median(groups).map(|s| {
             s.into_datetime(self.0.time_unit(), self.0.time_zone().clone())
                 .into_series()
         })
     }
-    #[cfg(feature = "lazy")]
-    fn agg_valid_count(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
-        self.0.agg_valid_count(groups)
-    }
-    fn hash_join_inner(&self, other: &Series) -> Vec<(u32, u32)> {
-        let other = other.to_physical_repr().into_owned();
-        self.0.hash_join_inner(other.as_ref().as_ref())
-    }
-    fn hash_join_left(&self, other: &Series) -> Vec<(u32, Option<u32>)> {
-        let other = other.to_physical_repr().into_owned();
-        self.0.hash_join_left(other.as_ref().as_ref())
-    }
-    fn hash_join_outer(&self, other: &Series) -> Vec<(Option<u32>, Option<u32>)> {
-        let other = other.to_physical_repr().into_owned();
-        self.0.hash_join_outer(other.as_ref().as_ref())
-    }
     fn zip_outer_join_column(
         &self,
         right_column: &Series,
-        opt_join_tuples: &[(Option<u32>, Option<u32>)],
+        opt_join_tuples: &[(Option<IdxSize>, Option<IdxSize>)],
     ) -> Series {
         let right_column = right_column.to_physical_repr().into_owned();
         self.0
@@ -256,16 +206,26 @@ impl private::PrivateSeries for SeriesWrap<DatetimeChunked> {
             "cannot do remainder operation on logical".into(),
         ))
     }
-    fn group_tuples(&self, multithreaded: bool) -> GroupTuples {
-        self.0.group_tuples(multithreaded)
+    fn group_tuples(&self, multithreaded: bool, sorted: bool) -> GroupsProxy {
+        self.0.group_tuples(multithreaded, sorted)
     }
     #[cfg(feature = "sort_multiple")]
-    fn argsort_multiple(&self, by: &[Series], reverse: &[bool]) -> Result<UInt32Chunked> {
+    fn argsort_multiple(&self, by: &[Series], reverse: &[bool]) -> Result<IdxCa> {
         self.0.deref().argsort_multiple(by, reverse)
     }
 }
 
 impl SeriesTrait for SeriesWrap<DatetimeChunked> {
+    fn is_sorted(&self) -> IsSorted {
+        if self.0.is_sorted() {
+            IsSorted::Ascending
+        } else if self.0.is_sorted_reverse() {
+            IsSorted::Descending
+        } else {
+            IsSorted::Not
+        }
+    }
+
     #[cfg(feature = "interpolate")]
     fn interpolate(&self) -> Series {
         self.0
@@ -318,8 +278,8 @@ impl SeriesTrait for SeriesWrap<DatetimeChunked> {
 
     fn append(&mut self, other: &Series) -> Result<()> {
         if self.0.dtype() == other.dtype() {
-            let other = other.to_physical_repr().into_owned();
-            self.0.append(other.as_ref().as_ref());
+            let other = other.to_physical_repr();
+            self.0.append(other.as_ref().as_ref().as_ref());
             Ok(())
         } else {
             Err(PolarsError::SchemaMisMatch(
@@ -328,6 +288,18 @@ impl SeriesTrait for SeriesWrap<DatetimeChunked> {
         }
     }
 
+    fn extend(&mut self, other: &Series) -> Result<()> {
+        if self.0.dtype() == other.dtype() {
+            let other = other.to_physical_repr();
+            self.0.extend(other.as_ref().as_ref().as_ref());
+            Ok(())
+        } else {
+            Err(PolarsError::SchemaMisMatch(
+                "cannot extend Series; data types don't match".into(),
+            ))
+        }
+    }
+
     fn filter(&self, filter: &BooleanChunked) -> Result<Series> {
         self.0.filter(filter).map(|ca| {
             ca.into_datetime(self.0.time_unit(), self.0.time_zone().clone())
@@ -335,7 +307,21 @@ impl SeriesTrait for SeriesWrap<DatetimeChunked> {
         })
     }
 
-    fn take(&self, indices: &UInt32Chunked) -> Result<Series> {
+    #[cfg(feature = "chunked_ids")]
+    unsafe fn _take_chunked_unchecked(&self, by: &[ChunkId]) -> Series {
+        let ca = self.0.deref().take_chunked_unchecked(by);
+        ca.into_datetime(self.0.time_unit(), self.0.time_zone().clone())
+            .into_series()
+    }
+
+    #[cfg(feature = "chunked_ids")]
+    unsafe fn _take_opt_chunked_unchecked(&self, by: &[Option<ChunkId>]) -> Series {
+        let ca = self.0.deref().take_opt_chunked_unchecked(by);
+        ca.into_datetime(self.0.time_unit(), self.0.time_zone().clone())
+            .into_series()
+    }
+
+    fn take(&self, indices: &IdxCa) -> Result<Series> {
         ChunkTake::take(self.0.deref(), indices.into()).map(|ca| {
             ca.into_datetime(self.0.time_unit(), self.0.time_zone().clone())
                 .into_series()
@@ -362,7 +348,7 @@ impl SeriesTrait for SeriesWrap<DatetimeChunked> {
             .into_series()
     }
 
-    unsafe fn take_unchecked(&self, idx: &UInt32Chunked) -> Result<Series> {
+    unsafe fn take_unchecked(&self, idx: &IdxCa) -> Result<Series> {
         Ok(ChunkTake::take_unchecked(self.0.deref(), idx.into())
             .into_datetime(self.0.time_unit(), self.0.time_zone().clone())
             .into_series())
@@ -401,48 +387,10 @@ impl SeriesTrait for SeriesWrap<DatetimeChunked> {
     }
 
     fn cast(&self, data_type: &DataType) -> Result<Series> {
-        const NS_IN_DAY: i64 = 86_400_000_000_000;
-        const MS_IN_DAY: i64 = 86400000;
-        use DataType::*;
-        let ca = match (self.dtype(), data_type) {
-            (Datetime(TimeUnit::Milliseconds, _), Datetime(TimeUnit::Nanoseconds, tz)) => {
-                return Ok((self.0.as_ref() * 1_000_000i64)
-                    .into_datetime(TimeUnit::Nanoseconds, tz.clone())
-                    .into_series())
-            }
-            (Datetime(TimeUnit::Nanoseconds, _), Datetime(TimeUnit::Milliseconds, tz)) => {
-                return Ok((self.0.as_ref() / 1_000_000i64)
-                    .into_datetime(TimeUnit::Milliseconds, tz.clone())
-                    .into_series())
-            }
-            #[cfg(feature = "dtype-date")]
-            (Datetime(tu, _), Date) => match tu {
-                TimeUnit::Nanoseconds => {
-                    return Ok((self.0.as_ref() / NS_IN_DAY)
-                        .cast(&Int32)
-                        .unwrap()
-                        .into_date()
-                        .into_series());
-                }
-                TimeUnit::Milliseconds => {
-                    return Ok((self.0.as_ref() / MS_IN_DAY)
-                        .cast(&Int32)
-                        .unwrap()
-                        .into_date()
-                        .into_series());
-                }
-            },
-            _ => Cow::Borrowed(self.0.deref()),
-        };
-        ca.cast(data_type)
-    }
-
-    fn to_dummies(&self) -> Result<DataFrame> {
-        self.0.to_dummies()
-    }
-
-    fn value_counts(&self) -> Result<DataFrame> {
-        self.0.value_counts()
+        match data_type {
+            DataType::Utf8 => Ok(self.0.strftime("%F %T").into_series()),
+            _ => self.0.cast(data_type),
+        }
     }
 
     fn get(&self, index: usize) -> AnyValue {
@@ -450,6 +398,7 @@ impl SeriesTrait for SeriesWrap<DatetimeChunked> {
     }
 
     #[inline]
+    #[cfg(feature = "private")]
     unsafe fn get_unchecked(&self, index: usize) -> AnyValue {
         self.0
             .get_any_value_unchecked(index)
@@ -463,8 +412,8 @@ impl SeriesTrait for SeriesWrap<DatetimeChunked> {
             .into_series()
     }
 
-    fn argsort(&self, reverse: bool) -> UInt32Chunked {
-        self.0.argsort(reverse)
+    fn argsort(&self, options: SortOptions) -> IdxCa {
+        self.0.argsort(options)
     }
 
     fn null_count(&self) -> usize {
@@ -486,7 +435,7 @@ impl SeriesTrait for SeriesWrap<DatetimeChunked> {
         self.0.n_unique()
     }
 
-    fn arg_unique(&self) -> Result<UInt32Chunked> {
+    fn arg_unique(&self) -> Result<IdxCa> {
         self.0.arg_unique()
     }
 
@@ -554,11 +503,6 @@ impl SeriesTrait for SeriesWrap<DatetimeChunked> {
             .min_as_series()
             .into_datetime(self.0.time_unit(), self.0.time_zone().clone())
     }
-    fn mean_as_series(&self) -> Series {
-        Int32Chunked::full_null(self.name(), 1)
-            .cast(self.dtype())
-            .unwrap()
-    }
     fn median_as_series(&self) -> Series {
         Int32Chunked::full_null(self.name(), 1)
             .cast(self.dtype())
@@ -604,7 +548,7 @@ impl SeriesTrait for SeriesWrap<DatetimeChunked> {
         self.0.is_in(other)
     }
     #[cfg(feature = "repeat_by")]
-    fn repeat_by(&self, by: &UInt32Chunked) -> ListChunked {
+    fn repeat_by(&self, by: &IdxCa) -> ListChunked {
         self.0
             .repeat_by(by)
             .cast(&DataType::List(Box::new(DataType::Datetime(
