@@ -15,6 +15,8 @@ mod floats;
 mod list;
 #[cfg(feature = "object")]
 mod object;
+#[cfg(feature = "dtype-struct")]
+mod struct_;
 mod utf8;
 
 #[cfg(feature = "object")]
@@ -23,20 +25,21 @@ use std::any::Any;
 use super::private;
 use super::IntoSeries;
 use super::SeriesTrait;
+use super::*;
 use crate::chunked_array::comparison::*;
 #[cfg(feature = "rolling_window")]
 use crate::chunked_array::ops::rolling_window::RollingOptions;
 use crate::chunked_array::{
     ops::{
-        aggregate::{ChunkAggSeries, VarAggSeries},
+        aggregate::{ChunkAggSeries, QuantileAggSeries, VarAggSeries},
         compare_inner::{IntoPartialEqInner, IntoPartialOrdInner, PartialEqInner, PartialOrdInner},
         explode::ExplodeByOffsets,
     },
-    AsSinglePtr, ChunkIdIter,
+    AsSinglePtr,
 };
 use crate::fmt::FmtList;
 use crate::frame::groupby::*;
-use crate::frame::hash_join::{HashJoin, ZipOuterJoinColumn};
+use crate::frame::hash_join::ZipOuterJoinColumn;
 use crate::prelude::*;
 #[cfg(feature = "checked_arithmetic")]
 use crate::series::arithmetic::checked::NumOpsDispatchChecked;
@@ -136,11 +139,6 @@ macro_rules! impl_dyn_series {
                 self.0.cummin(reverse).into_series()
             }
 
-            #[cfg(feature = "asof_join")]
-            fn join_asof(&self, other: &Series) -> Result<Vec<Option<u32>>> {
-                self.0.join_asof(other)
-            }
-
             fn set_sorted(&mut self, reverse: bool) {
                 self.0.set_sorted(reverse)
             }
@@ -174,19 +172,15 @@ macro_rules! impl_dyn_series {
                 self.0.vec_hash_combine(build_hasher, hashes)
             }
 
-            fn agg_mean(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
-                self.0.agg_mean(groups)
-            }
-
-            fn agg_min(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_min(&self, groups: &GroupsProxy) -> Option<Series> {
                 self.0.agg_min(groups)
             }
 
-            fn agg_max(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_max(&self, groups: &GroupsProxy) -> Option<Series> {
                 self.0.agg_max(groups)
             }
 
-            fn agg_sum(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_sum(&self, groups: &GroupsProxy) -> Option<Series> {
                 use DataType::*;
                 match self.dtype() {
                     Int8 | UInt8 | Int16 | UInt16 => self.cast(&Int64).unwrap().agg_sum(groups),
@@ -194,60 +188,34 @@ macro_rules! impl_dyn_series {
                 }
             }
 
-            fn agg_first(&self, groups: &[(u32, Vec<u32>)]) -> Series {
-                self.0.agg_first(groups)
-            }
-
-            fn agg_last(&self, groups: &[(u32, Vec<u32>)]) -> Series {
-                self.0.agg_last(groups)
-            }
-
-            fn agg_std(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_std(&self, groups: &GroupsProxy) -> Option<Series> {
                 self.0.agg_std(groups)
             }
 
-            fn agg_var(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_var(&self, groups: &GroupsProxy) -> Option<Series> {
                 self.0.agg_var(groups)
             }
 
-            fn agg_n_unique(&self, groups: &[(u32, Vec<u32>)]) -> Option<UInt32Chunked> {
-                self.0.agg_n_unique(groups)
-            }
-
-            fn agg_list(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_list(&self, groups: &GroupsProxy) -> Option<Series> {
                 self.0.agg_list(groups)
             }
 
             fn agg_quantile(
                 &self,
-                groups: &[(u32, Vec<u32>)],
+                groups: &GroupsProxy,
                 quantile: f64,
                 interpol: QuantileInterpolOptions,
             ) -> Option<Series> {
                 self.0.agg_quantile(groups, quantile, interpol)
             }
 
-            fn agg_median(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_median(&self, groups: &GroupsProxy) -> Option<Series> {
                 self.0.agg_median(groups)
             }
-            #[cfg(feature = "lazy")]
-            fn agg_valid_count(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
-                self.0.agg_valid_count(groups)
-            }
-
-            fn hash_join_inner(&self, other: &Series) -> Vec<(u32, u32)> {
-                HashJoin::hash_join_inner(&self.0, other.as_ref().as_ref())
-            }
-            fn hash_join_left(&self, other: &Series) -> Vec<(u32, Option<u32>)> {
-                HashJoin::hash_join_left(&self.0, other.as_ref().as_ref())
-            }
-            fn hash_join_outer(&self, other: &Series) -> Vec<(Option<u32>, Option<u32>)> {
-                HashJoin::hash_join_outer(&self.0, other.as_ref().as_ref())
-            }
             fn zip_outer_join_column(
                 &self,
                 right_column: &Series,
-                opt_join_tuples: &[(Option<u32>, Option<u32>)],
+                opt_join_tuples: &[(Option<IdxSize>, Option<IdxSize>)],
             ) -> Series {
                 ZipOuterJoinColumn::zip_outer_join_column(&self.0, right_column, opt_join_tuples)
             }
@@ -266,17 +234,27 @@ macro_rules! impl_dyn_series {
             fn remainder(&self, rhs: &Series) -> Result<Series> {
                 NumOpsDispatch::remainder(&self.0, rhs)
             }
-            fn group_tuples(&self, multithreaded: bool) -> GroupTuples {
-                IntoGroupTuples::group_tuples(&self.0, multithreaded)
+            fn group_tuples(&self, multithreaded: bool, sorted: bool) -> GroupsProxy {
+                IntoGroupsProxy::group_tuples(&self.0, multithreaded, sorted)
             }
 
             #[cfg(feature = "sort_multiple")]
-            fn argsort_multiple(&self, by: &[Series], reverse: &[bool]) -> Result<UInt32Chunked> {
+            fn argsort_multiple(&self, by: &[Series], reverse: &[bool]) -> Result<IdxCa> {
                 self.0.argsort_multiple(by, reverse)
             }
         }
 
         impl SeriesTrait for SeriesWrap<$ca> {
+            fn is_sorted(&self) -> IsSorted {
+                if self.0.is_sorted() {
+                    IsSorted::Ascending
+                } else if self.0.is_sorted_reverse() {
+                    IsSorted::Descending
+                } else {
+                    IsSorted::Not
+                }
+            }
+
             #[cfg(feature = "rolling_window")]
             fn rolling_apply(
                 &self,
@@ -500,7 +478,6 @@ macro_rules! impl_dyn_series {
 
             fn append(&mut self, other: &Series) -> Result<()> {
                 if self.0.dtype() == other.dtype() {
-                    // todo! add object
                     self.0.append(other.as_ref().as_ref());
                     Ok(())
                 } else {
@@ -510,6 +487,17 @@ macro_rules! impl_dyn_series {
                 }
             }
 
+            fn extend(&mut self, other: &Series) -> Result<()> {
+                if self.0.dtype() == other.dtype() {
+                    self.0.extend(other.as_ref().as_ref());
+                    Ok(())
+                } else {
+                    Err(PolarsError::SchemaMisMatch(
+                        "cannot extend Series; data types don't match".into(),
+                    ))
+                }
+            }
+
             fn filter(&self, filter: &BooleanChunked) -> Result<Series> {
                 ChunkFilter::filter(&self.0, filter).map(|ca| ca.into_series())
             }
@@ -522,7 +510,17 @@ macro_rules! impl_dyn_series {
                 self.0.median()
             }
 
-            fn take(&self, indices: &UInt32Chunked) -> Result<Series> {
+            #[cfg(feature = "chunked_ids")]
+            unsafe fn _take_chunked_unchecked(&self, by: &[ChunkId]) -> Series {
+                self.0.take_chunked_unchecked(by).into_series()
+            }
+
+            #[cfg(feature = "chunked_ids")]
+            unsafe fn _take_opt_chunked_unchecked(&self, by: &[Option<ChunkId>]) -> Series {
+                self.0.take_opt_chunked_unchecked(by).into_series()
+            }
+
+            fn take(&self, indices: &IdxCa) -> Result<Series> {
                 let indices = if indices.chunks.len() > 1 {
                     Cow::Owned(indices.rechunk())
                 } else {
@@ -543,7 +541,7 @@ macro_rules! impl_dyn_series {
                 ChunkTake::take_unchecked(&self.0, iter.into()).into_series()
             }
 
-            unsafe fn take_unchecked(&self, idx: &UInt32Chunked) -> Result<Series> {
+            unsafe fn take_unchecked(&self, idx: &IdxCa) -> Result<Series> {
                 let idx = if idx.chunks.len() > 1 {
                     Cow::Owned(idx.rechunk())
                 } else {
@@ -577,19 +575,12 @@ macro_rules! impl_dyn_series {
                 self.0.cast(data_type)
             }
 
-            fn to_dummies(&self) -> Result<DataFrame> {
-                ToDummies::to_dummies(&self.0)
-            }
-
-            fn value_counts(&self) -> Result<DataFrame> {
-                ChunkUnique::value_counts(&self.0)
-            }
-
             fn get(&self, index: usize) -> AnyValue {
                 self.0.get_any_value(index)
             }
 
             #[inline]
+            #[cfg(feature = "private")]
             unsafe fn get_unchecked(&self, index: usize) -> AnyValue {
                 self.0.get_any_value_unchecked(index)
             }
@@ -598,8 +589,8 @@ macro_rules! impl_dyn_series {
                 ChunkSort::sort_with(&self.0, options).into_series()
             }
 
-            fn argsort(&self, reverse: bool) -> UInt32Chunked {
-                ChunkSort::argsort(&self.0, reverse)
+            fn argsort(&self, options: SortOptions) -> IdxCa {
+                ChunkSort::argsort(&self.0, options)
             }
 
             fn null_count(&self) -> usize {
@@ -618,7 +609,7 @@ macro_rules! impl_dyn_series {
                 ChunkUnique::n_unique(&self.0)
             }
 
-            fn arg_unique(&self) -> Result<UInt32Chunked> {
+            fn arg_unique(&self) -> Result<IdxCa> {
                 ChunkUnique::arg_unique(&self.0)
             }
 
@@ -671,11 +662,8 @@ macro_rules! impl_dyn_series {
             fn min_as_series(&self) -> Series {
                 ChunkAggSeries::min_as_series(&self.0)
             }
-            fn mean_as_series(&self) -> Series {
-                ChunkAggSeries::mean_as_series(&self.0)
-            }
             fn median_as_series(&self) -> Series {
-                ChunkAggSeries::median_as_series(&self.0)
+                QuantileAggSeries::median_as_series(&self.0)
             }
             fn var_as_series(&self) -> Series {
                 VarAggSeries::var_as_series(&self.0)
@@ -688,7 +676,7 @@ macro_rules! impl_dyn_series {
                 quantile: f64,
                 interpol: QuantileInterpolOptions,
             ) -> Result<Series> {
-                ChunkAggSeries::quantile_as_series(&self.0, quantile, interpol)
+                QuantileAggSeries::quantile_as_series(&self.0, quantile, interpol)
             }
 
             fn fmt_list(&self) -> String {
@@ -725,7 +713,7 @@ macro_rules! impl_dyn_series {
                 IsIn::is_in(&self.0, other)
             }
             #[cfg(feature = "repeat_by")]
-            fn repeat_by(&self, by: &UInt32Chunked) -> ListChunked {
+            fn repeat_by(&self, by: &IdxCa) -> ListChunked {
                 RepeatBy::repeat_by(&self.0, by)
             }
 
@@ -802,4 +790,16 @@ impl_dyn_series_numeric!(Int64Chunked);
 
 impl private::PrivateSeriesNumeric for SeriesWrap<Utf8Chunked> {}
 impl private::PrivateSeriesNumeric for SeriesWrap<ListChunked> {}
-impl private::PrivateSeriesNumeric for SeriesWrap<BooleanChunked> {}
+impl private::PrivateSeriesNumeric for SeriesWrap<BooleanChunked> {
+    fn bit_repr_is_large(&self) -> bool {
+        false
+    }
+    fn bit_repr_small(&self) -> UInt32Chunked {
+        self.0
+            .cast(&DataType::UInt32)
+            .unwrap()
+            .u32()
+            .unwrap()
+            .clone()
+    }
+}
