@@ -3,7 +3,10 @@ pub use crate::prelude::ChunkCompare;
 use crate::prelude::*;
 use arrow::array::ArrayRef;
 use polars_arrow::prelude::QuantileInterpolOptions;
+#[cfg(any(feature = "dtype-struct", feature = "object"))]
+use std::any::Any;
 
+mod any_value;
 pub(crate) mod arithmetic;
 mod comparison;
 mod from;
@@ -18,26 +21,26 @@ pub mod unstable;
 use crate::chunked_array::ops::rolling_window::RollingOptions;
 #[cfg(feature = "rank")]
 use crate::prelude::unique::rank::rank;
-#[cfg(feature = "groupby_list")]
-use crate::utils::Wrap;
 use crate::utils::{split_ca, split_series};
+use crate::utils::{split_offsets, Wrap};
 use crate::{series::arithmetic::coerce_lhs_rhs, POOL};
-#[cfg(feature = "groupby_list")]
 use ahash::RandomState;
+use arrow::compute::aggregate::estimated_bytes_size;
 pub use from::*;
 use num::NumCast;
 use rayon::prelude::*;
 pub use series_trait::*;
 use std::borrow::Cow;
-#[cfg(feature = "groupby_list")]
 use std::hash::{Hash, Hasher};
 use std::ops::Deref;
 use std::sync::Arc;
 
+pub use series_trait::IsSorted;
+
 /// # Series
 /// The columnar data type for a DataFrame.
 ///
-/// Most of the available functions are definedin the [SeriesTrait trait](crate::series::SeriesTrait).
+/// Most of the available functions are defined in the [SeriesTrait trait](crate::series::SeriesTrait).
 ///
 /// The `Series` struct consists
 /// of typed [ChunkedArray](../chunked_array/struct.ChunkedArray.html)'s. To quickly cast
@@ -55,7 +58,7 @@ use std::sync::Arc;
 /// You can do standard arithmetic on series.
 /// ```
 /// # use polars_core::prelude::*;
-/// let s: Series = [1, 2, 3].iter().collect();
+/// let s = Series::new("a", [1 , 2, 3]);
 /// let out_add = &s + &s;
 /// let out_sub = &s - &s;
 /// let out_div = &s / &s;
@@ -83,7 +86,7 @@ use std::sync::Arc;
 /// ```
 /// # use polars_core::prelude::*;
 /// let s = Series::new("dollars", &[1, 2, 3]);
-/// let mask = s.equal(1);
+/// let mask = s.equal(1).unwrap();
 /// let valid = [true, false, false].iter();
 /// assert!(mask
 ///     .into_iter()
@@ -118,8 +121,8 @@ use std::sync::Arc;
 ///
 /// ```
 /// # use polars_core::prelude::*;
-/// // Series van be created from Vec's, slices and arrays
-/// Series::new("boolean series", &vec![true, false, true]);
+/// // Series can be created from Vec's, slices and arrays
+/// Series::new("boolean series", &[true, false, true]);
 /// Series::new("int series", &[1, 2, 3]);
 /// // And can be nullable
 /// Series::new("got nulls", &[Some(1), None, Some(2)]);
@@ -134,26 +137,28 @@ use std::sync::Arc;
 #[must_use]
 pub struct Series(pub Arc<dyn SeriesTrait>);
 
-#[cfg(feature = "groupby_list")]
 impl PartialEq for Wrap<Series> {
     fn eq(&self, other: &Self) -> bool {
         self.0.series_equal_missing(other)
     }
 }
 
-#[cfg(feature = "groupby_list")]
 impl Eq for Wrap<Series> {}
 
-#[cfg(feature = "groupby_list")]
 impl Hash for Wrap<Series> {
     fn hash<H: Hasher>(&self, state: &mut H) {
         let rs = RandomState::with_seeds(0, 0, 0, 0);
-        let h = UInt64Chunked::new_from_aligned_vec("", self.0.vec_hash(rs)).sum();
+        let h = UInt64Chunked::from_vec("", self.0.vec_hash(rs)).sum();
         h.hash(state)
     }
 }
 
 impl Series {
+    /// Create a new empty Series
+    pub fn new_empty(name: &str, dtype: &DataType) -> Series {
+        Series::full_null(name, 0, dtype)
+    }
+
     pub(crate) fn get_inner_mut(&mut self) -> &mut dyn SeriesTrait {
         if Arc::weak_count(&self.0) + Arc::strong_count(&self.0) != 1 {
             self.0 = self.0.clone_inner();
@@ -161,6 +166,10 @@ impl Series {
         Arc::get_mut(&mut self.0).expect("implementation error")
     }
 
+    pub fn into_frame(self) -> DataFrame {
+        DataFrame::new_no_checks(vec![self])
+    }
+
     /// Rename series.
     pub fn rename(&mut self, name: &str) -> &mut Series {
         self.get_inner_mut().rename(name);
@@ -178,12 +187,22 @@ impl Series {
         Ok(self)
     }
 
-    /// Append a Series of the same type in place.
+    /// Append in place. This is done by adding the chunks of `other` to this [`Series`].
+    ///
+    /// See [`ChunkedArray::append`] and [`ChunkedArray::extend`].
     pub fn append(&mut self, other: &Series) -> Result<&mut Self> {
         self.get_inner_mut().append(other)?;
         Ok(self)
     }
 
+    /// Extend the memory backed by this array with the values from `other`.
+    ///
+    /// See [`ChunkedArray::extend`] and [`ChunkedArray::append`].
+    pub fn extend(&mut self, other: &Series) -> Result<&mut Self> {
+        self.get_inner_mut().extend(other)?;
+        Ok(self)
+    }
+
     pub fn sort(&self, reverse: bool) -> Self {
         self.sort_with(SortOptions {
             descending: reverse,
@@ -341,27 +360,14 @@ impl Series {
         match self.dtype() {
             Date => Cow::Owned(self.cast(&DataType::Int32).unwrap()),
             Datetime(_, _) | Duration(_) | Time => Cow::Owned(self.cast(&DataType::Int64).unwrap()),
-            Categorical => Cow::Owned(self.cast(&DataType::UInt32).unwrap()),
+            #[cfg(feature = "dtype-categorical")]
+            Categorical(_) => Cow::Owned(self.cast(&DataType::UInt32).unwrap()),
             _ => Cow::Borrowed(self),
         }
     }
 
-    /// Take by index if ChunkedArray contains a single chunk.
-    ///
-    /// # Safety
-    /// This doesn't check any bounds. Null validity is checked.
-    pub unsafe fn take_unchecked_threaded(
-        &self,
-        idx: &UInt32Chunked,
-        rechunk: bool,
-    ) -> Result<Series> {
-        let n_threads = POOL.current_num_threads();
-        let idx = split_ca(idx, n_threads)?;
-
-        let series: Result<Vec<_>> =
-            POOL.install(|| idx.par_iter().map(|idx| self.take_unchecked(idx)).collect());
-
-        let s = series?
+    fn finish_take_threaded(&self, s: Vec<Series>, rechunk: bool) -> Series {
+        let s = s
             .into_iter()
             .reduce(|mut s, s1| {
                 s.append(&s1).unwrap();
@@ -369,39 +375,83 @@ impl Series {
             })
             .unwrap();
         if rechunk {
-            Ok(s.rechunk())
+            s.rechunk()
         } else {
-            Ok(s)
+            s
         }
     }
 
+    // take a function pointer to reduce bloat
+    fn threaded_op(
+        &self,
+        rechunk: bool,
+        len: usize,
+        func: &(dyn Fn(usize, usize) -> Result<Series> + Send + Sync),
+    ) -> Result<Series> {
+        let n_threads = POOL.current_num_threads();
+        let offsets = split_offsets(len, n_threads);
+
+        let series: Result<Vec<_>> = POOL.install(|| {
+            offsets
+                .into_par_iter()
+                .map(|(offset, len)| func(offset, len))
+                .collect()
+        });
+
+        Ok(self.finish_take_threaded(series?, rechunk))
+    }
+
+    /// Take by index if ChunkedArray contains a single chunk.
+    ///
+    /// # Safety
+    /// This doesn't check any bounds. Null validity is checked.
+    pub unsafe fn take_unchecked_threaded(&self, idx: &IdxCa, rechunk: bool) -> Result<Series> {
+        self.threaded_op(rechunk, idx.len(), &|offset, len| {
+            let idx = idx.slice(offset as i64, len);
+            self.take_unchecked(&idx)
+        })
+    }
+
+    /// # Safety
+    /// This doesn't check any bounds. Null validity is checked.
+    #[cfg(feature = "chunked_ids")]
+    pub(crate) unsafe fn _take_chunked_unchecked_threaded(
+        &self,
+        chunk_ids: &[ChunkId],
+        rechunk: bool,
+    ) -> Series {
+        self.threaded_op(rechunk, chunk_ids.len(), &|offset, len| {
+            let chunk_ids = &chunk_ids[offset..offset + len];
+            Ok(self._take_chunked_unchecked(chunk_ids))
+        })
+        .unwrap()
+    }
+
+    /// # Safety
+    /// This doesn't check any bounds. Null validity is checked.
+    #[cfg(feature = "chunked_ids")]
+    pub(crate) unsafe fn _take_opt_chunked_unchecked_threaded(
+        &self,
+        chunk_ids: &[Option<ChunkId>],
+        rechunk: bool,
+    ) -> Series {
+        self.threaded_op(rechunk, chunk_ids.len(), &|offset, len| {
+            let chunk_ids = &chunk_ids[offset..offset + len];
+            Ok(self._take_opt_chunked_unchecked(chunk_ids))
+        })
+        .unwrap()
+    }
+
     /// Take by index. This operation is clone.
     ///
     /// # Safety
     ///
     /// Out of bounds access doesn't Error but will return a Null value
-    pub fn take_threaded(&self, idx: &UInt32Chunked, rechunk: bool) -> Result<Series> {
-        let n_threads = POOL.current_num_threads();
-        let idx = split_ca(idx, n_threads).unwrap();
-
-        let series = POOL.install(|| {
-            idx.par_iter()
-                .map(|idx| self.take(idx))
-                .collect::<Result<Vec<_>>>()
-        })?;
-
-        let s = series
-            .into_iter()
-            .reduce(|mut s, s1| {
-                s.append(&s1).unwrap();
-                s
-            })
-            .unwrap();
-        if rechunk {
-            Ok(s.rechunk())
-        } else {
-            Ok(s)
-        }
+    pub fn take_threaded(&self, idx: &IdxCa, rechunk: bool) -> Result<Series> {
+        self.threaded_op(rechunk, idx.len(), &|offset, len| {
+            let idx = idx.slice(offset as i64, len);
+            self.take(&idx)
+        })
     }
 
     /// Filter by boolean mask. This operation clones data.
@@ -424,18 +474,7 @@ impl Series {
                 .collect()
         });
 
-        let s = series?
-            .into_iter()
-            .reduce(|mut s, s1| {
-                s.append(&s1).unwrap();
-                s
-            })
-            .unwrap();
-        if rechunk {
-            Ok(s.rechunk())
-        } else {
-            Ok(s)
-        }
+        Ok(self.finish_take_threaded(series?, rechunk))
     }
 
     #[cfg(feature = "dot_product")]
@@ -448,7 +487,7 @@ impl Series {
     #[cfg_attr(docsrs, doc(cfg(feature = "row_hash")))]
     /// Get a hash of this Series
     pub fn hash(&self, build_hasher: ahash::RandomState) -> UInt64Chunked {
-        UInt64Chunked::new_from_aligned_vec(self.name(), self.0.vec_hash(build_hasher))
+        UInt64Chunked::from_vec(self.name(), self.0.vec_hash(build_hasher))
     }
 
     /// Get the sum of the Series as a new Series of length 1.
@@ -668,7 +707,7 @@ impl Series {
         }
     }
     /// Apply a rolling median to a Series. See:
-    /// [ChunkedArray::rolling_median](crate::prelude::ChunkWindow::rolling_median).
+    /// [`ChunkedArray::rolling_median`]
     #[cfg_attr(docsrs, doc(cfg(feature = "rolling_window")))]
     pub fn rolling_median(&self, _options: RollingOptions) -> Result<Series> {
         #[cfg(feature = "rolling_window")]
@@ -681,7 +720,7 @@ impl Series {
         }
     }
     /// Apply a rolling quantile to a Series. See:
-    /// [ChunkedArray::rolling_quantile](crate::prelude::ChunkWindow::rolling_quantile).
+    /// [`ChunkedArray::rolling_quantile`]
     #[cfg_attr(docsrs, doc(cfg(feature = "rolling_window")))]
     pub fn rolling_quantile(
         &self,
@@ -751,6 +790,10 @@ impl Series {
 
     #[cfg(feature = "dtype-time")]
     pub(crate) fn into_time(self) -> Series {
+        #[cfg(not(feature = "dtype-time"))]
+        {
+            panic!("activate feature dtype-time")
+        }
         match self.dtype() {
             DataType::Int64 => self.i64().unwrap().clone().into_time().into_series(),
             DataType::Time => self
@@ -765,6 +808,10 @@ impl Series {
     }
 
     pub(crate) fn into_date(self) -> Series {
+        #[cfg(not(feature = "dtype-date"))]
+        {
+            panic!("activate feature dtype-date")
+        }
         match self.dtype() {
             #[cfg(feature = "dtype-date")]
             DataType::Int32 => self.i32().unwrap().clone().into_date().into_series(),
@@ -780,6 +827,11 @@ impl Series {
         }
     }
     pub(crate) fn into_datetime(self, timeunit: TimeUnit, tz: Option<TimeZone>) -> Series {
+        #[cfg(not(feature = "dtype-datetime"))]
+        {
+            panic!("activate feature dtype-datetime")
+        }
+
         match self.dtype() {
             #[cfg(feature = "dtype-datetime")]
             DataType::Int64 => self
@@ -801,6 +853,10 @@ impl Series {
     }
 
     pub(crate) fn into_duration(self, timeunit: TimeUnit) -> Series {
+        #[cfg(not(feature = "dtype-duration"))]
+        {
+            panic!("activate feature dtype-duration")
+        }
         match self.dtype() {
             #[cfg(feature = "dtype-duration")]
             DataType::Int64 => self
@@ -823,11 +879,7 @@ impl Series {
 
     /// Check if the underlying data is a logical type.
     pub fn is_logical(&self) -> bool {
-        use DataType::*;
-        matches!(
-            self.dtype(),
-            Date | Duration(_) | Datetime(_, _) | Time | Categorical
-        )
+        self.dtype().is_logical()
     }
 
     /// Check if underlying physical data is numeric.
@@ -844,25 +896,6 @@ impl Series {
         }
     }
 
-    /// Check if underlying data is numeric
-    pub fn is_numeric(&self) -> bool {
-        // allow because it cannot be replaced when object feature is activated
-        #[allow(clippy::match_like_matches_macro)]
-        match self.dtype() {
-            DataType::Utf8
-            | DataType::List(_)
-            | DataType::Categorical
-            | DataType::Date
-            | DataType::Datetime(_, _)
-            | DataType::Duration(_)
-            | DataType::Boolean
-            | DataType::Null => false,
-            #[cfg(feature = "object")]
-            DataType::Object(_) => false,
-            _ => true,
-        }
-    }
-
     #[cfg(feature = "abs")]
     #[cfg_attr(docsrs, doc(cfg(feature = "abs")))]
     /// convert numerical values to their absolute value
@@ -915,6 +948,89 @@ impl Series {
         };
         self.slice(-(len as i64), len)
     }
+
+    pub fn mean_as_series(&self) -> Series {
+        let val = [self.mean()];
+        let s = Series::new(self.name(), val);
+        if !self.dtype().is_numeric() {
+            s.cast(self.dtype()).unwrap()
+        } else {
+            s
+        }
+    }
+
+    /// Compute the unique elements, but maintain order. This requires more work
+    /// than a naive [`Series::unique`](SeriesTrait::unique).
+    pub fn unique_stable(&self) -> Result<Series> {
+        let idx = self.arg_unique()?;
+        // Safety:
+        // Indices are in bounds.
+        unsafe { self.take_unchecked(&idx) }
+    }
+
+    pub fn idx(&self) -> Result<&IdxCa> {
+        #[cfg(feature = "bigidx")]
+        {
+            self.u64()
+        }
+        #[cfg(not(feature = "bigidx"))]
+        {
+            self.u32()
+        }
+    }
+
+    /// Unpack to ChunkedArray of dtype struct
+    #[cfg(feature = "dtype-struct")]
+    pub fn struct_(&self) -> Result<&StructChunked> {
+        #[cfg(feature = "dtype-struct")]
+        match self.dtype() {
+            DataType::Struct(_) => {
+                let any = self.as_any();
+
+                debug_assert!(any.is::<StructChunked>());
+                // Safety
+                // We just checked type
+                Ok(unsafe { &*(any as *const dyn Any as *const StructChunked) })
+            }
+            _ => Err(PolarsError::SchemaMisMatch(
+                format!("Series dtype {:?} != struct", self.dtype()).into(),
+            )),
+        }
+    }
+
+    /// Returns an estimation of the total (heap) allocated size of the `Series` in bytes.
+    ///
+    /// # Implementation
+    /// This estimation is the sum of the size of its buffers, validity, including nested arrays.
+    /// Multiple arrays may share buffers and bitmaps. Therefore, the size of 2 arrays is not the
+    /// sum of the sizes computed from this function. In particular, [`StructArray`]'s size is an upper bound.
+    ///
+    /// When an array is sliced, its allocated size remains constant because the buffer unchanged.
+    /// However, this function will yield a smaller number. This is because this function returns
+    /// the visible size of the buffer, not its total capacity.
+    ///
+    /// FFI buffers are included in this estimation.
+    pub fn estimated_size(&self) -> usize {
+        #[allow(unused_mut)]
+        let mut size = self
+            .chunks()
+            .iter()
+            .map(|arr| estimated_bytes_size(&**arr))
+            .sum();
+        match self.dtype() {
+            #[cfg(feature = "dtype-categorical")]
+            DataType::Categorical(Some(rv)) => match &**rv {
+                RevMapping::Local(arr) => size += estimated_bytes_size(arr),
+                RevMapping::Global(map, arr, _) => {
+                    size +=
+                        map.capacity() * std::mem::size_of::<u32>() * 2 + estimated_bytes_size(arr);
+                }
+            },
+            _ => {}
+        }
+
+        size
+    }
 }
 
 impl Deref for Series {
