@@ -1,6 +1,5 @@
 #[cfg(feature = "object")]
 use crate::chunked_array::object::PolarsObjectSafe;
-use crate::chunked_array::ChunkIdIter;
 pub use crate::prelude::ChunkCompare;
 use crate::prelude::*;
 use arrow::array::ArrayRef;
@@ -8,7 +7,6 @@ use polars_arrow::prelude::QuantileInterpolOptions;
 use std::any::Any;
 use std::borrow::Cow;
 #[cfg(feature = "temporal")]
-use std::ops::Deref;
 use std::sync::Arc;
 
 #[derive(Debug, Copy, Clone)]
@@ -170,9 +168,6 @@ pub(crate) mod private {
         fn vec_hash_combine(&self, _build_hasher: RandomState, _hashes: &mut [u64]) {
             invalid_operation_panic!(self)
         }
-        fn agg_mean(&self, _groups: &GroupsProxy) -> Option<Series> {
-            None
-        }
         fn agg_min(&self, _groups: &GroupsProxy) -> Option<Series> {
             None
         }
@@ -489,10 +484,16 @@ pub trait SeriesTrait:
         invalid_operation_panic!(self)
     }
 
+    #[doc(hidden)]
+    #[cfg(feature = "chunked_ids")]
+    unsafe fn _take_chunked_unchecked(&self, by: &[ChunkId]) -> Series;
+
+    #[doc(hidden)]
+    #[cfg(feature = "chunked_ids")]
+    unsafe fn _take_opt_chunked_unchecked(&self, by: &[Option<ChunkId>]) -> Series;
+
     /// Take by index from an iterator. This operation clones the data.
-    fn take_iter(&self, _iter: &mut dyn TakeIterator) -> Result<Series> {
-        invalid_operation_panic!(self)
-    }
+    fn take_iter(&self, _iter: &mut dyn TakeIterator) -> Result<Series>;
 
     /// Take by index from an iterator. This operation clones the data.
     ///
@@ -500,17 +501,13 @@ pub trait SeriesTrait:
     ///
     /// - This doesn't check any bounds.
     /// - Iterator must be TrustedLen
-    unsafe fn take_iter_unchecked(&self, _iter: &mut dyn TakeIterator) -> Series {
-        invalid_operation_panic!(self)
-    }
+    unsafe fn take_iter_unchecked(&self, _iter: &mut dyn TakeIterator) -> Series;
 
     /// Take by index if ChunkedArray contains a single chunk.
     ///
     /// # Safety
     /// This doesn't check any bounds.
-    unsafe fn take_unchecked(&self, _idx: &IdxCa) -> Result<Series> {
-        invalid_operation_panic!(self)
-    }
+    unsafe fn take_unchecked(&self, _idx: &IdxCa) -> Result<Series>;
 
     /// Take by index from an iterator. This operation clones the data.
     ///
@@ -518,11 +515,10 @@ pub trait SeriesTrait:
     ///
     /// - This doesn't check any bounds.
     /// - Iterator must be TrustedLen
-    unsafe fn take_opt_iter_unchecked(&self, _iter: &mut dyn TakeIteratorNulls) -> Series {
-        invalid_operation_panic!(self)
-    }
+    unsafe fn take_opt_iter_unchecked(&self, _iter: &mut dyn TakeIteratorNulls) -> Series;
 
     /// Take by index from an iterator. This operation clones the data.
+    /// todo! remove?
     #[cfg(feature = "take_opt_iter")]
     #[cfg_attr(docsrs, doc(cfg(feature = "take_opt_iter")))]
     fn take_opt_iter(&self, _iter: &mut dyn TakeIteratorNulls) -> Result<Series> {
@@ -530,14 +526,10 @@ pub trait SeriesTrait:
     }
 
     /// Take by index. This operation is clone.
-    fn take(&self, _indices: &IdxCa) -> Result<Series> {
-        invalid_operation_panic!(self)
-    }
+    fn take(&self, _indices: &IdxCa) -> Result<Series>;
 
     /// Get length of series.
-    fn len(&self) -> usize {
-        invalid_operation_panic!(self)
-    }
+    fn len(&self) -> usize;
 
     /// Check if Series is empty.
     fn is_empty(&self) -> bool {
@@ -591,11 +583,6 @@ pub trait SeriesTrait:
         invalid_operation_panic!(self)
     }
 
-    /// Create dummy variables. See [DataFrame](DataFrame::to_dummies)
-    fn to_dummies(&self) -> Result<DataFrame> {
-        invalid_operation_panic!(self)
-    }
-
     /// Get a single value by index. Don't use this operation for loops as a runtime cast is
     /// needed for every iteration.
     fn get(&self, _index: usize) -> AnyValue {
@@ -808,198 +795,6 @@ pub trait SeriesTrait:
         "fmt implemented".into()
     }
 
-    #[cfg(feature = "temporal")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "temporal")))]
-    /// Extract hour from underlying NaiveDateTime representation.
-    /// Returns the hour number from 0 to 23.
-    fn hour(&self) -> Result<UInt32Chunked> {
-        match self.dtype() {
-            #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.hour()),
-            #[cfg(feature = "dtype-time")]
-            DataType::Time => self.time().map(|ca| ca.hour()),
-            _ => Err(PolarsError::InvalidOperation(
-                format!("operation not supported on dtype {:?}", self.dtype()).into(),
-            )),
-        }
-    }
-
-    #[cfg(feature = "temporal")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "temporal")))]
-    /// Extract minute from underlying NaiveDateTime representation.
-    /// Returns the minute number from 0 to 59.
-    fn minute(&self) -> Result<UInt32Chunked> {
-        match self.dtype() {
-            #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.minute()),
-            #[cfg(feature = "dtype-time")]
-            DataType::Time => self.time().map(|ca| ca.minute()),
-            _ => Err(PolarsError::InvalidOperation(
-                format!("operation not supported on dtype {:?}", self.dtype()).into(),
-            )),
-        }
-    }
-
-    #[cfg(feature = "temporal")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "temporal")))]
-    /// Extract second from underlying NaiveDateTime representation.
-    /// Returns the second number from 0 to 59.
-    fn second(&self) -> Result<UInt32Chunked> {
-        match self.dtype() {
-            #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.second()),
-            #[cfg(feature = "dtype-time")]
-            DataType::Time => self.time().map(|ca| ca.second()),
-            _ => Err(PolarsError::InvalidOperation(
-                format!("operation not supported on dtype {:?}", self.dtype()).into(),
-            )),
-        }
-    }
-
-    #[cfg(feature = "temporal")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "temporal")))]
-    /// Returns the number of nanoseconds since the whole non-leap second.
-    /// The range from 1,000,000,000 to 1,999,999,999 represents the leap second.
-    fn nanosecond(&self) -> Result<UInt32Chunked> {
-        match self.dtype() {
-            #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.nanosecond()),
-            #[cfg(feature = "dtype-time")]
-            DataType::Time => self.time().map(|ca| ca.nanosecond()),
-            _ => Err(PolarsError::InvalidOperation(
-                format!("operation not supported on dtype {:?}", self.dtype()).into(),
-            )),
-        }
-    }
-
-    #[cfg(feature = "temporal")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "temporal")))]
-    /// Extract day from underlying NaiveDateTime representation.
-    /// Returns the day of month starting from 1.
-    ///
-    /// The return value ranges from 1 to 31. (The last day of month differs by months.)
-    fn day(&self) -> Result<UInt32Chunked> {
-        match self.dtype() {
-            #[cfg(feature = "dtype-date")]
-            DataType::Date => self.date().map(|ca| ca.day()),
-            #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.day()),
-            _ => Err(PolarsError::InvalidOperation(
-                format!("operation not supported on dtype {:?}", self.dtype()).into(),
-            )),
-        }
-    }
-    #[cfg(feature = "temporal")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "temporal")))]
-    /// Returns the weekday number where monday = 0 and sunday = 6
-    fn weekday(&self) -> Result<UInt32Chunked> {
-        match self.dtype() {
-            #[cfg(feature = "dtype-date")]
-            DataType::Date => self.date().map(|ca| ca.weekday()),
-            #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.weekday()),
-            _ => Err(PolarsError::InvalidOperation(
-                format!("operation not supported on dtype {:?}", self.dtype()).into(),
-            )),
-        }
-    }
-
-    #[cfg(feature = "temporal")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "temporal")))]
-    /// Returns the ISO week number starting from 1.
-    /// The return value ranges from 1 to 53. (The last week of year differs by years.)
-    fn week(&self) -> Result<UInt32Chunked> {
-        match self.dtype() {
-            #[cfg(feature = "dtype-date")]
-            DataType::Date => self.date().map(|ca| ca.week()),
-            #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.week()),
-            _ => Err(PolarsError::InvalidOperation(
-                format!("operation not supported on dtype {:?}", self.dtype()).into(),
-            )),
-        }
-    }
-
-    #[cfg(feature = "temporal")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "temporal")))]
-    /// Returns the day of year starting from 1.
-    ///
-    /// The return value ranges from 1 to 366. (The last day of year differs by years.)
-    fn ordinal_day(&self) -> Result<UInt32Chunked> {
-        match self.dtype() {
-            #[cfg(feature = "dtype-date")]
-            DataType::Date => self.date().map(|ca| ca.ordinal()),
-            #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.ordinal()),
-            _ => Err(PolarsError::InvalidOperation(
-                format!("operation not supported on dtype {:?}", self.dtype()).into(),
-            )),
-        }
-    }
-
-    #[cfg(feature = "temporal")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "temporal")))]
-    /// Extract month from underlying NaiveDateTime representation.
-    /// Returns the month number starting from 1.
-    ///
-    /// The return value ranges from 1 to 12.
-    fn month(&self) -> Result<UInt32Chunked> {
-        match self.dtype() {
-            #[cfg(feature = "dtype-date")]
-            DataType::Date => self.date().map(|ca| ca.month()),
-            #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.month()),
-            _ => Err(PolarsError::InvalidOperation(
-                format!("operation not supported on dtype {:?}", self.dtype()).into(),
-            )),
-        }
-    }
-
-    #[cfg(feature = "temporal")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "temporal")))]
-    /// Extract month from underlying NaiveDateTime representation.
-    /// Returns the year number in the calendar date.
-    fn year(&self) -> Result<Int32Chunked> {
-        match self.dtype() {
-            #[cfg(feature = "dtype-date")]
-            DataType::Date => self.date().map(|ca| ca.year()),
-            #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.year()),
-            _ => Err(PolarsError::InvalidOperation(
-                format!("operation not supported on dtype {:?}", self.dtype()).into(),
-            )),
-        }
-    }
-
-    #[cfg(feature = "temporal")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "temporal")))]
-    /// Format Date/Datetimewith a `fmt` rule. See [chrono strftime/strptime](https://docs.rs/chrono/0.4.19/chrono/format/strftime/index.html).
-    fn strftime(&self, fmt: &str) -> Result<Series> {
-        match self.dtype() {
-            #[cfg(feature = "dtype-date")]
-            DataType::Date => self.date().map(|ca| ca.strftime(fmt).into_series()),
-            #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.strftime(fmt).into_series()),
-            #[cfg(feature = "dtype-time")]
-            DataType::Time => self.time().map(|ca| ca.strftime(fmt).into_series()),
-            _ => Err(PolarsError::InvalidOperation(
-                format!("operation not supported on dtype {:?}", self.dtype()).into(),
-            )),
-        }
-    }
-
-    #[cfg(all(
-        feature = "temporal",
-        feature = "dtype-date",
-        feature = "dtype-datetime"
-    ))]
-    #[cfg_attr(docsrs, doc(cfg(feature = "temporal")))]
-    /// Convert date(time) object to timestamp in [`TimeUnit`].
-    fn timestamp(&self, tu: TimeUnit) -> Result<Int64Chunked> {
-        self.cast(&DataType::Datetime(tu, None))
-            .map(|s| s.datetime().unwrap().deref().clone())
-    }
-
     /// Clone inner ChunkedArray and wrap in a new Arc
     fn clone_inner(&self) -> Arc<dyn SeriesTrait> {
         invalid_operation_panic!(self)
@@ -1018,6 +813,12 @@ pub trait SeriesTrait:
         invalid_operation_panic!(self)
     }
 
+    /// Get a hold to self as `Any` trait reference.
+    /// Only implemented for ObjectType
+    fn as_any_mut(&mut self) -> &mut dyn Any {
+        invalid_operation_panic!(self)
+    }
+
     /// Raise a numeric series to the power of exponent.
     fn pow(&self, _exponent: f64) -> Result<Series> {
         Err(PolarsError::InvalidOperation(
