@@ -3,7 +3,6 @@ pub(crate) mod series;
 use crate::prelude::*;
 use crate::POOL;
 pub use arrow;
-pub use num_cpus;
 pub use polars_arrow::utils::TrustMyLength;
 pub use polars_arrow::utils::*;
 pub use rayon;
@@ -110,6 +109,7 @@ where
 }
 
 // prefer this one over split_ca, as this can push the null_count into the thread pool
+#[doc(hidden)]
 pub fn split_offsets(len: usize, n: usize) -> Vec<(usize, usize)> {
     if n == 1 {
         vec![(0, len)]
@@ -131,11 +131,13 @@ pub fn split_offsets(len: usize, n: usize) -> Vec<(usize, usize)> {
 }
 
 #[cfg(feature = "private")]
+#[doc(hidden)]
 pub fn split_series(s: &Series, n: usize) -> Result<Vec<Series>> {
     split_array!(s, n, i64)
 }
 
 #[cfg(feature = "private")]
+#[doc(hidden)]
 pub fn split_df(df: &DataFrame, n: usize) -> Result<Vec<DataFrame>> {
     trait Len {
         fn len(&self) -> usize;
@@ -148,8 +150,14 @@ pub fn split_df(df: &DataFrame, n: usize) -> Result<Vec<DataFrame>> {
     split_array!(df, n, i64)
 }
 
+pub fn slice_slice<T>(vals: &[T], offset: i64, len: usize) -> &[T] {
+    let (raw_offset, slice_len) = slice_offsets(offset, len, vals.len());
+    &vals[raw_offset..raw_offset + slice_len]
+}
+
 #[inline]
 #[cfg(feature = "private")]
+#[doc(hidden)]
 pub fn slice_offsets(offset: i64, length: usize, array_len: usize) -> (usize, usize) {
     let abs_offset = offset.abs() as usize;
 
@@ -297,6 +305,7 @@ macro_rules! apply_method_all_arrow_series {
             DataType::Date => $self.date().unwrap().$method($($args),*),
             DataType::Datetime(_, _) => $self.datetime().unwrap().$method($($args),*),
             DataType::List(_) => $self.list().unwrap().$method($($args),*),
+            DataType::Struct(_) => $self.struct_().unwrap().$method($($args),*),
             dt => panic!("dtype {:?} not supported", dt)
         }
     }
@@ -894,6 +903,17 @@ pub(crate) unsafe fn copy_from_slice_unchecked<T>(src: &[T], dst: &mut [T]) {
     std::ptr::copy_nonoverlapping(src.as_ptr(), dst.as_mut_ptr(), dst.len());
 }
 
+#[cfg(feature = "chunked_ids")]
+pub(crate) fn create_chunked_index_mapping(chunks: &[ArrayRef], len: usize) -> Vec<ChunkId> {
+    let mut vals = Vec::with_capacity(len);
+
+    for (chunk_i, chunk) in chunks.iter().enumerate() {
+        vals.extend((0..chunk.len()).map(|array_i| [chunk_i as IdxSize, array_i as IdxSize]))
+    }
+
+    vals
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
@@ -923,22 +943,4 @@ mod test {
             b.chunk_id().collect::<Vec<_>>()
         );
     }
-
-    #[test]
-    fn test_df_macro_trailing_commas() -> Result<()> {
-        let a = df! {
-            "a" => &["a one", "a two"],
-            "b" => &["b one", "b two"],
-            "c" => &[1, 2]
-        }?;
-
-        let b = df! {
-            "a" => &["a one", "a two"],
-            "b" => &["b one", "b two"],
-            "c" => &[1, 2],
-        }?;
-
-        assert!(a.frame_equal(&b));
-        Ok(())
-    }
 }
